function [ gene ] = extract_transdefs_gtf( filename , gene_selection , annotation_extension )
	%HINT : Only 1 gene is returned, selected with the string stored in gene_selection. To change this change the return value " gene " to " genes "
	% genes is a cell array of that contains a type " gene " in each element. 
	% gene is a data structure made of multi layered cell arrays.  The first layer is a brief description of the gene and contains calculation-relevant values.
	% gene { 1 } is the ID of the gene
	% gene { 2 } is the number of transcripts for the gene
	% gene { 3 } is the number of exons of each transcript of the gene, in order of the transcripts contained in gene {6}
	% gene { 4 } is the length of each transcript ( in transcript coordinates without intron spanning distance ) in the same order.
	% gene { 5 } is the chromosome identifier as found in the .gff file.
	% gene { 6 } is the link to the next layer of cell arrays. This layer now contains data for each transcript. Transcript specific data is accessed by selecting the index of the desired transcript, for example to access the 1st  transcript { 6 } { 1 } .
	% gene { 6 } { 1 } { 1 } is the ID of the 1st transcript
	% gene { 6 } { 1 } { 2 } is the strandedness information of the 1st transcript
	% gene { 6 } { 1 } { 3 } is the extension status of the 1st transcript. A transcript definition may be extended via the annotation_extension value
	% gene { 6 } { 1 } { 4 } is a one dimensional matrix that holds the start and end positions of the annotated exons of the 1st transcript in alternating order 
	%Gene selection is a string that contains the ID of a gene that is to be returned. If no value was provided or 0 the first gene in the annotation list is returned 
	if nargin < 2
		gene_selection = 0;
	end
	return_index = 1;
	if nargin < 3
		annotation_extension = 0;
	end
	%File read
	fid=fopen(filename,"r");
	count_validline=0;
	EOFcount=0;
	while( EOFcount != 10 )
		s=fgetl(fid);
		if ( (s == -1) || (length(s)<1) )
			EOFcount ++;
		else
			EOFcount = 0;
			inread{++count_validline} = s;
		end
	end
	fclose(fid);
	genes=[];
	nr_genes = 0;
	nr_transcripts = 0;
	%File parsing
	for kk=1:1:count_validline
		lineanal=strsplit(inread{kk},"\t");	%Each line is split into its columns. 
		if !strcmp( lineanal{3} , "exon" )	%Exon info lines only
			continue
		end
		extra_fields=strsplit(lineanal{9},";");
		extra_fields(end)=[];	%Remove the last&empty field
		%Mandatory order of columns:
		%Chromosome Name, bio_info, entrytype, Exon Chr Start (bp), Exon Chr End (bp), qual, Strand, *, extra fields
		%extra fields: gene_id "ENSG00000167754"; transcript_id "ENST00000391809"; gene_name "KLK5"; gene_source "ensembl_havana"; gene_biotype "protein_coding"; transcript_name "KLK5-005"; transcript_source "ensembl_havana"; tag "CCDS"; ccds_id "CCDS12810";	
		%The extra fields are parsed for the gene id and the gene id and the associated gene name ( insert gene id if missing )	
		region = lineanal{1};
		strandedness= lineanal{7};
		ex_start = str2num(lineanal{4});
		ex_end = str2num(lineanal{5});
		gene_id=[];
		transcript_id=[];
		fallback=[];
		associated_gene_name=[];
		for ll = 1:length(extra_fields)
			extra_fields{ll} = deblank(extra_fields{ll});
			[fieldname,value] = sscanf(extra_fields{ll},"%s \"%s","C");
			value(end)=[];
			if strcmp(fieldname,"gene_id")
				gene_id=value;
				fallback=value;
			elseif strcmp(fieldname,"gene_name")
				associated_gene_name=value;
			elseif strcmp(fieldname,"transcript_id")
				transcript_id=value;
			end
		end
		if(isempty(associated_gene_name))
			associated_gene_name=fallback;
		end
		if !(temp_index_gene = whereis( genes , gene_id ))
			temp_index_gene = length(genes)+1;
			genes{temp_index_gene}{1} = gene_id;
			genes{temp_index_gene}{5} = region;
			genes{temp_index_gene}{6} = [];
		end
		if !(temp_index_trans = whereis( genes{temp_index_gene}{6} , transcript_id ))
			temp_index_trans = length(genes{temp_index_gene}{6})+1;
			genes{temp_index_gene}{6}{temp_index_trans}{1} = transcript_id;
			genes{temp_index_gene}{6}{temp_index_trans}{2} = strandedness;
			genes{temp_index_gene}{6}{temp_index_trans}{3} = "not extended";
			genes{temp_index_gene}{6}{temp_index_trans}{4} = [];					
		end
		%Fill the positions into the exon structure. Don't mind order etc.
		if( isempty(genes{temp_index_gene}{6}{temp_index_trans}{4}) )
			genes{temp_index_gene}{6}{temp_index_trans}{4}(1:2) = [ex_start,ex_end];
		else
			range = length(genes{temp_index_gene}{6}{temp_index_trans}{4});
			genes{temp_index_gene}{6}{temp_index_trans}{4}(range+1:range+2) = [ex_start,ex_end];
		end
	end
	%Fill up the missing meta data into appropriate fields ( trans_lengths and nr_transcripts ) and sort the order of the exons in each gene
	for ig = 1:1:length(genes)
		if((gene_selection&&(strcmp(genes{ig}{1},gene_selection))))
			return_index = ig;
		end
		genes{ig}{2} = length(genes{ig}{end});
		clear trans_lengths
		clear nr_exons
		for iat = 1:1:genes{ig}{2}
			genes{ig}{end}{iat}{end} = sort ( genes{ig}{end}{iat}{end} );
			nr_exons(iat) = length(genes{ig}{end}{iat}{end}) / 2;
			trans_lengths(iat) = sum(genes{ig}{end}{iat}{end}( 2:2:end ) .- genes{ig}{end}{iat}{end}( 1:2:end ) ) + nr_exons(iat);
		end
		genes{ig}{3} = nr_exons;
		genes{ig}{4} = trans_lengths;
	end
	% annotation_extension is a value that is used to extend the annotation of a number of transcripts in a gene. Extending the annotation will introduce a certain loss of information as the differentiation by difference in startsite/endsite of the transcripts is changed. This, however, is intended in case that the correctness of the annotation is questionable. For this, each transcript is enlonged each into 5' and 3' direction for the value of annotation_extension ( ID of transcript ). The routine uses the the internal ID hence the order of the transcripts in the gff file dictates which transcript is associated with which annotation_extension element. Any natural number that is positive and higher than 0 is accepted as extension value.
%The layout of the annotation_extension variable dictates which gene is affected by the extension. Only genes with number of transcripts identical to the number of elements in annotation_extension will be treated.
% A value of 0 is interpreted as intention to skip the extension procedure
%%%	WARNING : THE LENGTHS OF THE TRANSCRIPTS ARE NOT EXTENDED! %%%
	if any(annotation_extension)	
		for ig = 1:1:length(genes)
			if ( genes{ig}{2} == length( annotation_extension ) )
				for iat = 1:1:genes{ig}{2}
					genes{ig}{end}{iat}{end}(1) -= annotation_extension(iat);
					genes{ig}{end}{iat}{end}(end) += annotation_extension(iat);
					if annotation_extension(iat)
						genes{ig}{end}{iat}{3} = "extended";
					end
				end
			end
		end
	end
	gene = genes{return_index};
endfunction

function indexret = whereis( cellarraystructure , string_name )
	indexret = 0;
	for ii = 1 : length ( cellarraystructure )
		if strcmp( cellarraystructure{ii}{1} , string_name )
			indexret = ii;
			return
		end
	end
end

