%This function is a routine that operates the script mix2.m. mix2.m contains the calculation part of the mix2 model, while this script here covers pre-processing and post-processing. This includes the configuration of the model, the translation of the gene annotations, input processing, output and status reports. 
%This script should be operated via the scripts start_mix2_pairedend.osh and start_mix2_pairedend.m

function mix2_main( string_grouptyingfile , string_validityfile , string_previousparams , string_configfile, trans_defs_file , sample_file , nr_blocks , output_file , nr_initializations , nr_reads_experiment )
%%% ABORT DETECTION
% Minimal error handling

	if !exist(string_configfile,"file")
		error("Could not access file %s. This file was given as configuration file argument.",string_configfile);
	end
	if !exist(trans_defs_file,"file")
		error("Could not access file %s. This file was given as transcripts definition argument.",trans_defs_file);
	end
	if !exist(sample_file,"file")
		error("Could not access file %s. This file was given as aligned reads argument.",sample_file);
	end
	if ((nr_blocks==Inf) || (!isnumeric(nr_blocks)) || (nr_blocks<=1) || (nr_blocks==Inf) || (nr_blocks==NaN))
		error("Did not understand the number of blocks argument.");
	end
	if ((nr_initializations==Inf) || (!isnumeric(nr_initializations)) || (nr_initializations<=0) || (nr_initializations==Inf) || (nr_initializations==NaN))
		error("Did not understand the number of initializations argument.");
	end

	more off;
	format long;

%%% Declaration section
% This section fills inital values into a number of crucial variables
	AUTO_GROUPING = 0;
	AUTO_TYING_NUS = 0;
	SAVE_ALL_RESULTS = 0;	%This variable is either 0 or 1 ; If set to 1, the script will save each initialisation along to the final result .sav file. 
	groups_ranges = [];

	annotation_extension=0;
	nr_iterations=1000;

	%The update_* variables define the behavior of the Mix² model. A parameter is updated after each iteration of Mix² if the update_* variable with its name is set to 1. E.g. : alphas variable is updated if update_alphas is set to 1. 
	update_alphas = 1;
	update_betas = 1;
	update_mus = 0;
	update_sigmas = 0;
	update_nus = 0;
	update_lambdas = 0;

	%The variables min_log_like_diff and min_par_diff are part of the termination conditions for the Mix² model
	min_log_like_diff = 0.5;
	min_par_diff = 1e-9;
	
	%The variable min_frags_per_group is a threshold which has to be exceeded by the count of valid fragments per transcripts of a group
	min_frags_per_group = 20;
	max_difference_starts = 50;

	%The *_init_var variables define the initialisation of the parameters for the Mix² model. A parameter is initialised with its default value. This value can change if a configuration file has another value stored for the parameter. The last change immediately before the start of the estimation is the 'variation of parameters'. This variation is configured with the *_init_var variables. Any parameter with a variation variable with value 0 ( e.g. alphas_init_var = 0 ) is not varied during the variation of parameters step. Other values result into a random change depending on the value of the parameter and the value of the annotation extension matrix. 
	alphas_init_var=0;
	betas_init_var=0;
	mus_init_var=0;
	sigmas_init_var=0;
	lambdas_init_var=0;
	nus_init_var=0;

	pl_params_mu = 200;		%This is the default mean value for the pdf p ( l(r) | s(r) , t )
	pl_params_sigma = 80;	%This is the default standard variation value for the pdf p ( l(r) | s(r) , t )

%This is the first import of variables from the config file.
	load(string_configfile);
	printf("loaded %s\n",string_configfile);
	if !exist("var","gene_selection")
		gene_selection = 0;
	end	

%The software creates a "result" file for each initialisation. If the SAVE_ALL_RESULTS variable is set to 1 each of these result files is permanently saved with a number e.g. the third initialisation is marked with the number 3. 
	[ifs_dir, ifs_name, ifs_ext] = fileparts(output_file);
	if(ifs_dir)
		result_bname = sprintf("%s/%s", ifs_dir, ifs_name);
	else
		result_bname = ifs_name;
	end
	result_ext = ifs_ext;

%Import the gene definition(s)
	[fdir,fname,fend] = fileparts(trans_defs_file);	%It is possible to provide gene annotations either as .gtf or .gff.
	if strcmp(fend,".gff")
		[ gene ] = extract_transdefs_gff( trans_defs_file , gene_selection , 0 );
	elseif strcmp(fend,".gtf")
		[ gene ] = extract_transdefs_gtf( trans_defs_file , gene_selection , 0 );
	else
		error("did not recognize the file type of the gene/transcript annotation\n");
	end
	gene_id = gene{1};
	nr_transcripts = gene{2};
	nr_exons = gene{3};	
	trans_lengths = gene{4};
	region = gene{5};
		
	for iat=1:nr_transcripts
		trans_identity{iat} = gene{6}{iat}{1};
		trans_start(iat) = gene{end}{iat}{end}(1);
	end

%Default values for the trained parameters
	[max_len, max_len_ind] = max(trans_lengths); 
	alphas = 1/nr_transcripts*ones(1,nr_transcripts);	%Abundance estimator values
	mus = ([1:nr_blocks] * max_len/nr_blocks) - (max_len/(2 * nr_blocks) );
	sigmas = ones(1,nr_blocks) * max_len / ( 2 * nr_blocks );
	nus = zeros(1,nr_transcripts);
	lambdas = trans_lengths/max_len;

%Annotation extension & calculation of transcript specific startsites, fragment lengths and validity matrices
	if !strcmp(string_validityfile,"")
		load(string_validityfile); 
		printf("loaded %s\n",string_validityfile);
	end;	%Loading validities if the argument -v was provided
	
	if sum(annotation_extension) %If any of the transcripts is to be extended
		printf("annotation_extension detected: %d\n", min(annotation_extension));
		if  length(annotation_extension) != nr_transcripts	%If the annotation_extension variable is not a matrix with 1 row and nr_transcripts-times columns, this is corrected here.
			annotation_extension(1:nr_transcripts) = annotation_extension;
		end
		for iat=1:nr_transcripts
			%The nus and the trans_lengths are adjusted here. The annotation itself is extended right before the reads are extracted.
			nus(iat) = annotation_extension(iat);
			trans_lengths(iat) += 2* annotation_extension(iat);
			trans_start(iat) -= annotation_extension(iat);
			gene{end}{iat}{3} = sprintf("extended for %d",annotation_extension(iat));
			gene{end}{iat}{end}(1) -= annotation_extension(iat);
			gene{end}{iat}{end}(end) += annotation_extension(iat);	
		end
		if exist("fragment_startsites_trans_coord","var") && exist("fragment_lengths_trans_coord","var") && exist("fragment_validity_mat","var")
			printf("the valdity matrix & the observations are not calculated anew\n");
		end
	end

%Check for existing fragment_startsites_trans_coord , fragment_lengths_trans_coord and fragment_validity_mat matrices
	if exist("fragment_startsites_trans_coord","var") && exist("fragment_lengths_trans_coord","var") && exist("fragment_validity_mat","var")
		printf("fss, fls and validity matrices exist\n");
	else
		printf("fss, fls and validity matrices have to be calculated...\n");
		tic;
		[ fragment_startsites_trans_coord, fragment_startsites_validity_mat , fragment_lengths_trans_coord , fragment_lengths_validity_mat , nr_reads_locus ] = validity_starts_lengths( trans_defs_file , sample_file , annotation_extension );
		printf("finished calculating the matrices, elapsed time: %.3fs\n",toc);
	end
	nr_fragments = size( fragment_startsites_trans_coord , 1 )
	temp_fss(:,1,:) = fragment_startsites_trans_coord;
	temp_fssv(:,1,:) = fragment_startsites_validity_mat;
	temp_fl(:,1,:) = fragment_lengths_trans_coord;
	temp_flv(:,1,:) = fragment_lengths_validity_mat;

	if isempty(fragment_startsites_validity_mat)
		error("No fragment was valid")
	end

%Initialize the group defining variables, then check if a group order was provided or if AUTO grouping was enabled
	array_groupid = ones(1,nr_transcripts);
	if !strcmp(string_grouptyingfile,"")
		s=load(string_grouptyingfile);
		printf("loaded %s\n",string_grouptyingfile);
		if(isstruct(s))
			array_groupid=struct2cell(s){1};
		else
			array_groupid=s;
		end
	elseif(AUTO_GROUPING)
		printf("AUTO_GROUPING option enabled! I will group the transcripts according to the ouput of tie_trans_length.m\n");

		max_nr_groups = nr_transcripts;
		if isempty(groups_ranges)
			groups_cell = tie_trans_length( trans_lengths, max_nr_groups, sum(fragment_startsites_validity_mat,1), min_frags_per_group );
		else	
			groups_cell = tie_trans_length( trans_lengths, max_nr_groups, sum(fragment_startsites_validity_mat,1), min_frags_per_group , 1 , groups_ranges );
		end

		for gid=1:length(groups_cell)
			array_groupid( groups_cell{gid} ) = gid;
		end

	end

	nr_groups=max(array_groupid)	%The number of groups is equal to the maximum value present in the array_groupid variable. The variable itself is used to iterate through the groups

% Here is the 2nd load of the config file. It is possible that this file also holds values for the trained parameters, hence after inserting the default values these values would be overwritten here. 
	load(string_configfile);
	printf("loaded %s\n",string_configfile);

%Check for content in the string_previousparams string. These values are intended to overwrite the default values and the values of the configuration (if any present), if a file was provided.
	if !strcmp(string_previousparams,"")
		load(string_previousparams); 
		printf("loaded %s\n",string_previousparams);
	end;	%Loading a result file if the argument -r was provided
	
	if exist("iteration_results_alphas","var")
		if(size(iteration_results_alphas,2) == nr_groups)
			%all parameters are formatted as cell array that hold the values separated by group id
			iteration_results_alphas = iteration_results_alphas{1};
		end	
		alphas = iteration_results_alphas{length(iteration_results_alphas)};	%Overwriting the alphas variable
		for gid=1:nr_groups
			betas{gid} = iteration_results_betas{gid}{length(iteration_results_betas{gid})};	%Overwriting the betas variable
		end
		%	The remaining result information is discarded
		clear iteration_results_alphas;
		clear iteration_results_betas;
		clear iteration_results_mus;
		clear iteration_results_sigmas;
		clear iteration_results_nus;
		clear iteration_results_lambdas;
	else
		%Setting betas in case that there was no configuration or anything else that has set them already
		for gid=1:nr_groups
			betas{gid} = 1/nr_blocks*ones(1,nr_blocks);
		end
	end

%Update adjustments
	if (length(update_lambdas) != nr_transcripts) update_lambdas(1:nr_transcripts) = update_lambdas(1); end
	if (length(update_nus) != nr_transcripts) update_nus(1:nr_transcripts) = update_nus(1); end

	nus_pairing = [1:nr_transcripts];
	lambdas_pairing = [1:nr_transcripts];

	if AUTO_TYING_NUS
		gstarts = tie_trans_start( trans_start , max_difference_starts );
		for gid=1:length(gstarts)
			nus_pairing(gstarts{gid})=gid;
		end
	end

%LOG
%The standard output / the log file will contain these variables. It is possible to remove this section or parts of this section to reduce the output without further influence on the rest of the code.
	printf("number of block functions used = ")
	printf("%d",nr_blocks)
	printf("\n")
	printf("Isoform IDs = ")
	for iat=1:nr_transcripts
		printf("%s ",trans_identity{iat})
	end
	printf("\n")
	printf("Initial alphas = ")
	printf("%.3f    ",alphas)
	printf("\n")
	for gid=1:nr_groups
		printf("Initial betas group { %d } = ",gid)
		printf("%.3f    ",betas{gid})
		printf("\n")
	end
	printf("Initial mus = ")
	printf("%.3f    ",mus)
	printf("\n")
	printf("Initial sigmas = ")
	printf("%.3f    ",sigmas)
	printf("\n")
	printf("Initial nus = ")
	printf("%.3f    ",nus)
	printf("\n")
	printf("Initial lambdas = ")
	printf("%.3f    ",lambdas)
	printf("\n")
	printf("Training status (0 : not trained, 1 : trained) of alphas = ")
	printf("%d ",update_alphas)
	printf("\n")
	printf("Training status (0 : not trained, 1 : trained) of betas = ")
	printf("%d   ",update_betas)
	printf("\n")
	printf("Training status (0 : not trained, 1 : trained) of mus = ")
	printf("%d ",update_mus)
	printf("\n")
	printf("Training status (0 : not trained, 1 : trained) of sigmas = ")
	printf("%d ",update_sigmas)
	printf("\n")
	printf("Training status (0 : not trained, 1 : trained) of nus = ")
	printf("%d ",update_nus)
	printf("\n")
	printf("Training status (0 : not trained, 1 : trained) of lambdas = ")
	printf("%d ",update_lambdas)
	printf("\n")

	printf("Tying IDs of nus = ")
	printf("%d ", nus_pairing)
	printf("\n")
	printf("Tying IDs of lambdas = ")
	printf("%d ", lambdas_pairing)
	printf("\n")

	printf("Variation scaling for each initialisation for the alphas = ")
	printf("%.6f    ",alphas_init_var)
	printf("\n")
	printf("Variation scaling for each initialisation for the betas = ")
	printf("%.6f   ",betas_init_var)
	printf("\n")
	printf("Variation scaling for each initialisation for the mus = ")
	printf("%.6f    ",mus_init_var)
	printf("\n")
	printf("Variation scaling for each initialisation for the sigmas = ")
	printf("%.6f    ",sigmas_init_var)
	printf("\n")
	printf("Variation scaling for each initialisation for the nus = ")
	printf("%.6f    ",nus_init_var)
	printf("\n")
	printf("Variation scaling for each initialisation for the lambdas = ")
	printf("%.6f    ",lambdas_init_var)
	printf("\n")
	printf("Threshold for minimum of difference in sum(log(likelihood)) between two iterations required to CONTINUE the estimation process = ")
	printf("%.9f",min_log_like_diff)
	printf("\n")
	printf("Threshold for minimum of difference of the parameter values between two iterations required to CONTINUE the estimation process = ")
	printf("%.9f",min_par_diff)
	printf("\n")
	printf("Group IDs of the isoforms = ")
	printf("%d ",array_groupid)
	printf("\n")
	printf("Assumed mean of the distribution of fragment lengths = ")
	printf("%f", pl_params_mu)
	printf("\n")
	printf("Assumed standard deviation of the distribution of fragment lengths = ")
	printf("%f", pl_params_sigma)
	printf("\n")
	fflush(stdout);
%/LOG

	%preparing the .sav file information
	for iat=1:nr_transcripts
		for ien = 1:(length(gene{end}{iat}{end})/2)
			annotations{iat}{ien}(1) = gene{end}{iat}{end}(ien*2-1);
			annotations{iat}{ien}(2) = gene{end}{iat}{end}(ien*2) ;
		end
	end

	tic
%	This is the start of the initialisations loop. This for loop iterates as often as specified in the nr_initializations variable. The *_init variables are the "parameter" input for the mix2.m script. Each of these variables may be variied here if specified in the configuration
	for iinit=1:nr_initializations
		%ALPHAS VARIATION
		do
			alphas_init = (alphas_init_var.*randn(1,nr_transcripts) + 1).*alphas;
		until (sum(alphas_init<0)<1)	%No negative values allowed
		alphas_init /= sum(alphas_init);	%Have to sum up to 1
		%BETAS VARIATION
		do
			for gid=1:nr_groups
				betas_init{gid} = (betas_init_var.*randn(1,nr_blocks)+1).*betas{gid};
				betas_pins(gid,:)=(betas_init{gid}<0);
			end
		until (sum(sum(betas_pins))<1)	%No negative values allowed
		for gid=1:nr_groups
			betas_init{gid} /= sum(betas_init{gid});	%Each set of betas have to sum up to 1
		end
		%MUS VARIATION
		mus_init = mus_init_var.*randn(1,nr_blocks)+mus;
		%SIGMAS VARIATION
		do
			sigmas_init = (sigmas_init_var.*randn(1,nr_blocks) + 1).*sigmas;
		until(sum(sigmas_init<0)<1)	%No negative values allowed

		%NUS VARIATION
		nus_init = (nus_init_var.*randn(1,nr_transcripts)+1).*nus;
		%LAMBDAS VARIATION
		do
			temp_lengths = max_len .* lambdas;
			init_lengths = randn(1,nr_transcripts) .* lambdas_init_var .* annotation_extension .+ temp_lengths;
			lambdas_init = init_lengths/max_len;
		until(sum(lambdas_init<0)<1)	%No negative values allowed

		[fid,temp_fname,msg] = mkstemp("./temp/octave-fpkm-em_XXXXXX",1);
		fclose(fid);
		tic;

%LOG
%The standard output / the log file will contain these variables. It is possible to remove this section or parts of this section to reduce the output without further influence on the rest of the code.
		printf("\n\nStarted Initialization %d at %s",iinit,ctime (time ()));
		printf("Initialization %d: Iteration 1 alphas  = ",iinit)
		printf("%.3f    ",alphas_init)
		printf("\n")
		for gid=1:nr_groups
			printf("Initialization %d: Iteration 1 betas group { %d } = ",iinit,gid)
			printf("%.3f    ",betas_init{gid})
			printf("\n")
		end
		printf("Initialization %d: Iteration 1 mus = ",iinit)
		printf("%.3f    ",mus_init)
		printf("\n")
		printf("Initialization %d: Iteration 1 sigmas = ",iinit)
		printf("%.3f    ",sigmas_init)
		printf("\n")
		printf("Initialization %d: Iteration 1 nus = ",iinit)
		printf("%.3f    ",nus_init)
		printf("\n")
		printf("Initialization %d: Iteration 1 lambdas = ",iinit)
		printf("%.3f    ",lambdas_init)
		printf("\n")
		fflush(stdout);
%/LOG

		[alphas_new , betas_new , mus_new , sigmas_new, nus_new , lambdas_new , total_model_log_like , iteration_finished , returnmessage] = mix2( temp_fname, alphas_init, betas_init, mus_init, sigmas_init, nus_init, lambdas_init, update_alphas, update_betas, update_mus, update_sigmas, update_nus, update_lambdas, min_log_like_diff, min_par_diff, array_groupid, nr_transcripts, nr_blocks, nr_fragments, trans_lengths, nr_iterations, temp_fss, temp_fssv, temp_fl, temp_flv, pl_params_mu, pl_params_sigma , nus_pairing , lambdas_pairing );

%LOG
%The standard output / the log file will contain these variables. It is possible to remove this section or parts of this section to reduce the output without further influence on the rest of the code.
		printf("\nFinished Initialization %d in iteration %d with log likelihood %f \n", iinit , iteration_finished , total_model_log_like );
		printf("%s\n",returnmessage);
		printf("Initialization %d: Final alphas  = ",iinit)
		printf("%.3f    ",alphas_new)
		printf("\n")
		for gid=1:nr_groups
			printf("Initialization %d: Final betas group { %d } = ",iinit,gid)
			printf("%.3f    ",betas_new{gid})
			printf("\n")
		end
		printf("Initialization %d: Final mus = ",iinit)
		printf("%.3f    ",mus_new)
		printf("\n")
		printf("Initialization %d: Final sigmas = ",iinit)
		printf("%.3f    ",sigmas_new)
		printf("\n")
		printf("Initialization %d: Final nus = ",iinit)
		printf("%.3f    ",nus_new)
		printf("\n")
		printf("Initialization %d: Final lambdas = ",iinit)
		printf("%.3f    ",lambdas_new)
		printf("\n")
		printf("Elapsed time of the initialization: %.3fs\n",toc);
		fflush(stdout);
%/LOG

		if SAVE_ALL_RESULTS
			result_file = sprintf("%s.nr_init_%03d%s",result_bname,iinit,result_ext);
			load(temp_fname);
			save("-text",output_file,"iteration_results_alphas","iteration_results_betas","iteration_results_mus","iteration_results_sigmas","iteration_results_lambdas","iteration_results_nus","iteration_results_total_model_log_like","array_groupid","groups_cell","gene","annotations","nr_fragments","nr_reads_experiment");
		end

		if ( (!isnan(total_model_log_like)) && ( !exist('total_model_log_like_max','var') || (total_model_log_like > total_model_log_like_max) ) )
			total_model_log_like_max = total_model_log_like;
			total_model_log_like_max_ind = iinit;
			alphas_max = alphas_new;
			betas_max = betas_new;
			mus_max = mus_new;
			sigmas_max = sigmas_new;
			nus_max = nus_new;
			lambdas_max = lambdas_new;
			load(temp_fname);
			save("-text",output_file,"iteration_results_alphas","iteration_results_betas","iteration_results_mus","iteration_results_sigmas","iteration_results_lambdas","iteration_results_nus","iteration_results_total_model_log_like","array_groupid","groups_cell","gene","annotations","nr_fragments","nr_reads_experiment");
		end
		unlink(temp_fname);
	end

%LOG
%The standard output / the log file will contain these variables. It is possible to remove this section or parts of this section to reduce the output without further influence on the rest of the code.
	printf("\nmaximal log likelihood %f found for initialization %d\n",total_model_log_like_max,total_model_log_like_max_ind);
	printf("Isoform IDs = ")
	for iat=1:nr_transcripts
		printf("%s ", trans_identity{iat})
	end
	printf("\n")
	printf("alphas = ")
	printf("%.3f    ",alphas_max)
	printf("\n")
	for gid=1:nr_groups
		printf("betas group { %d } = ",gid)
		printf("%.3f    ",betas_max{gid})
		printf("\n")
	end
	printf("mus = ")
	printf("%.3f    ",mus_max)
	printf("\n")
	printf("sigmas = ")
	printf("%.3f    ",sigmas_max)
	printf("\n")
	printf("nus = ")
	printf("%.3f    ",nus_max)
	printf("\n")
	printf("lambdas = ")
	printf("%.3f    ",lambdas_max)
	printf("\n")
	printf("END\n");
%/LOG

	trackfilename = sprintf("%s/%s.csv",ifs_dir, ifs_name);

	%Summary file
	fid=fopen ( trackfilename , 'w') ;
	fprintf(fid,"tracking_id\tgene_id\tInternal_ID\tlocus\tlength\tfragment_validity_coverage\tabundance\tnr_reads_locus\tnr_reads_experiment\tFPKM\n")
	for iat = 1 : nr_transcripts
		fprintf(fid,"%s\t%s\t%d\t%s:%d-%d\t%d\t%.2f\t%.5f\t%d\t%d\t%.5f\n",trans_identity{iat} , gene_id ,iat , region , gene{end}{iat}{end}(1) , gene{end}{iat}{end}(end) , trans_lengths(iat) , sum ( fragment_startsites_validity_mat (:,iat) ) / nr_fragments , alphas_max(iat) , nr_reads_locus , nr_reads_experiment , (nr_reads_locus * alphas_max(iat) * 10^9)/(trans_lengths(iat)*nr_reads_experiment) );
	end
	fclose(fid);
	
	printf("Stored OCTAVE/MATLAB's estimation results in %s\n",output_file);
	printf("Stored the summary of the estimation in %s\n",trackfilename);

	fflush(stdout);

